<?php
// $Id$

/**
 * * Fetch nodes for the selected date, or current date if none selected.
 *
 * @param $year
 *   Number of year
 * @param $month
 *   Number of month   
 * @param $day
 *   Number of day
 * @return
 *   A string with the themed page
 */
function archive_page($type = 'all', $year = 0, $month = 0, $day = 0) {
  // Check that there's nodes we can 
  $nodes = db_result(db_query(db_rewrite_sql('SELECT COUNT(1) FROM {node} n WHERE n.status = 1 AND n.type IN (\''. implode('\',\'', variable_get('archive_type_filters', array())) .'\')')));
  if (!$nodes && arg(1) == url('archive')) {
    return t('No content');
  }
  
  drupal_add_css(drupal_get_path('module', 'archive') .'/archive.css');
  $date = _archive_date($type, $year, $month, $day);
  $output = theme('archive_navigation', $type, $date);
  $nodes = variable_get('default_nodes_main', 10);
  $query = _archive_query($type, $date);
  $result = pager_query(db_rewrite_sql(array_shift($query)), $nodes, 0, NULL, $query);

  $found_rows = FALSE;
  $node_date = 0;
  while ($o = db_fetch_object($result)) {
    $node = node_load($o->nid);
    $node_created = $node->created + $date->tz;
    // Determine which separators are needed
    $separators = array('year'  => 0,
                        'month' => 0,
                        'day'   => 0);
    if (!$year) {
      $created_year = date('Y', $node_created);
      $last_year = date('Y', $node_date);
      if ($created_year != $last_year) {
        $separators['year'] = 1;
      }
    }
    // Print month separaters
    if (!$month) {
      $created_month = date('n', $node_created);
      $last_month = date('n', $node_date);
      if ($created_month != $last_month) {
        $separators['month'] = 1;
      }
    }
    // Print day separaters
    if (!$day) {
      $created_day = date('j', $node_created);
      $last_day = date('j', $node_date);
      if ($created_day != $last_day) {
        $separators['day'] = 1;
      }
    }
    $output .= theme('archive_separator', $node_created, $separators);
    $output .= node_view($node, TRUE);
    $found_rows = TRUE;
    $node_date = $node->created + $date->tz;
  }
  if ($found_rows) {
    $output .= theme('pager', NULL, $nodes);
  }
  // Handle URLs that are incorrectly typed and try to parse info out of them
  else {
    if ($date->days[$date->day]) {
      drupal_goto(_archive_url($type, $date->year, $date->month, $date->day));
    }
    else if ($date->months[$date->month]) {
      drupal_goto(_archive_url($type, $date->year, $date->month));
    }
    else if ($date->years[$date->year]) {
      drupal_goto(_archive_url($type, $date->year));
    }
    else {
      drupal_goto(_archive_url($type));
    }
  }

  return $output;
}

/**
 * Parses the current URL and populates an archive
 * date object with the selected date information.
 *
 * @param $year
 *   Number of year
 * @param $month
 *   Number of month
 * @param $day
 *   Number of day
 * @return
 *   A date object with GMT date values and a timezone value
 */
function _archive_date($type, $year = 0, $month = 0, $day = 0) {
  $date = new stdClass();
  $date->tz = _archive_get_timezone();

  $date->year  = 0;
  $date->month = 0;
  $date->day   = 0;
  
  if (_archive_validate_date($year, $month, $day)) {
    $date->year = $year;
    $date->month = $month;
    $date->day = $day;
  }
  else if (_archive_validate_date($year, $month)) {
    $date->year = $year;
    $date->month = $month;
  }
  else if (_archive_validate_date($year)) {
    $date->year = $year;
  }
  
  $post_counts = _archive_post_count($type, $date);
  $date->years  = $post_counts['years'];
  $date->months = $post_counts['months'];
  $date->days   = $post_counts['days'];
  
  return $date;
}

/**
 * Builds an SQL query array (query and parameters) to use
 * to display an archive page for the specified date.
 *
 * @param $date
 *    A date object obtained from _archive_date()
 * @return
 *    An array of (query, param_start, param_end)
 */
function _archive_query($type, $date) {
  // Confine the display interval to only one day
  if ($date->day) {
    $start = gmmktime(0, 0, 0, $date->month, $date->day, $date->year);
    $end   = gmmktime(0, 0, 0, $date->month, $date->day + 1, $date->year);
  }
  // Confine the display interval to one month
  else if ($date->month) {
    $start = gmmktime(0, 0, 0, $date->month, 1, $date->year);
    $end   = gmmktime(0, 0, 0, $date->month + 1, 1, $date->year);
  }
  // Confine the display interval to one year
  else if ($date->year) {
    $start = gmmktime(0, 0, 0, 1, 1, $date->year);
    $end   = gmmktime(0, 0, 0, 1, 1, $date->year + 1);
  }
  else {
    $start = 0;
    $end = 0;
  }
  
  // Grab limits on node types if exist
  $final_types = _archive_types_sql_string($type);
  
  // Allow viewing all nodes, not just nodes by year
  if ($start && $end) {
    return array('SELECT n.nid, n.type FROM {node} n WHERE n.status = 1 '. $final_types .'AND n.created >= %d AND n.created < %d ORDER BY n.created DESC', $start - $date->tz, $end - $date->tz);
  }
  else {
    return array('SELECT n.nid, n.type FROM {node} n WHERE n.status = 1 '. $final_types .'ORDER BY n.created DESC');
  }
}

function _archive_types_sql_string($type) {
  // Validate type and specify node types to include
  $final_types = '';
  if (_archive_validate_type($type) && $type != 'all') {
    $final_types = $type; 
  }
  else {
    $types = variable_get('archive_type_filters', array());
    // If no checkboxes selected
    if (!array_key_exists('0', $types)) {
      foreach ($types as $key => $value) {
        if (!$value) {
          unset($types[$key]);
        }
      }
      $final_types = implode('", "', array_keys($types));
    }
  }
  if (strlen($final_types) > 0) {
    $final_types = 'AND n.type IN ("'. $final_types .'") ';
  }
  return $final_types;
}

/**
 * Returns the range of dates with nodes.
 *
 * @param $type
 *    The object that is being filtered
 * @param $date
 *    A date object returned from _archive_date()
 * @return
 *    An array of the (first year with posts, last year with posts).
 */
function _archive_post_count($type, $date) {
  $final_types = _archive_types_sql_string($type);
  $result = db_fetch_object(db_query(db_rewrite_sql('SELECT MIN(n.created) AS min_date, MAX(n.created) AS max_date FROM {node} n WHERE n.status = 1 '. $final_types)));
  $mod = '';
  $with_posts = array('years' => array(), 'months' => array(), 'days' => array());
  $node_query = db_query(db_rewrite_sql('SELECT n.uid, n.created FROM {node} n WHERE n.status = 1 '. $final_types));
  while ($o = db_fetch_object($node_query)) {
    $node_date = explode(' ', format_date($o->created, 'custom', 'Y n j'));
    $with_posts['years'][$node_date[0]]++;
    if ($node_date[0] == $date->year) {
      $with_posts['months'][$node_date[1]]++;
      if ($node_date[1] == $date->month) {
        $with_posts['days'][$node_date[2]]++;
      }
    }
  }
  return $with_posts;
}

/**
 * Returns the different node types that have nodes
 *
 * @param $date
 *    A date object obtained from _archive_date()
 * @return
 *    An array of node-types to number of posts of that type
 */
function _archive_node_types($date) {
  
  $types = variable_get('archive_type_filters', array());
  
  // Confine the display interval to only one day
  if ($date->day) {
    $start = gmmktime(0, 0, 0, $date->month, $date->day, $date->year);
    $end   = gmmktime(0, 0, 0, $date->month, $date->day + 1, $date->year);
  }
  // Confine the display interval to one month
  else if ($date->month) {
    $start = gmmktime(0, 0, 0, $date->month, 1, $date->year);
    $end   = gmmktime(0, 0, 0, $date->month + 1, 1, $date->year);
  }
  // Confine the display interval to one year
  else if ($date->year) {
    $start = gmmktime(0, 0, 0, 1, 1, $date->year);
    $end   = gmmktime(0, 0, 0, 1, 1, $date->year + 1);
  }
  else {
    $start = 0;
    $end = 0;
  }
  if ($start && $end) {
    $result = db_query(db_rewrite_sql('SELECT t.type, t.name, COUNT(n.nid) AS node_count FROM {node} n INNER JOIN {node_type} t ON t.type = n.type WHERE n.status = 1 AND t.type IN ("'. join($types, '", "') .'") AND n.created BETWEEN %d AND %d GROUP BY n.type ORDER BY n.created'), $start - $date->tz, $end - $date->tz);
  }
  else {
    $result = db_query(db_rewrite_sql('SELECT t.type, t.name, COUNT(n.nid) AS node_count FROM {node} n INNER JOIN {node_type} t ON t.type = n.type WHERE n.status = 1 AND t.type IN ("'. join($types, '", "') .'") GROUP BY n.type ORDER BY n.created'));
  }
  
  $n_types = array();
  while ($row = db_fetch_array($result)) {
    $n_types[$row['type']] = array('count' => $row['node_count'],
                                   'name'  => $row['name']);
  }
  
  ksort($n_types);
  return $n_types;
}

/**
 * Check if given year, month and date combination
 * is valid for a Drupal archive.
 *
 * @param $year
 *    The year to check
 * @param $month
 *    The month to check
 * @param $day
 *    The day to check
 * @return
 *    TRUE or FALSE
 */
function _archive_validate_date($year, $month = NULL, $day = NULL) {
  $valid = FALSE;
  if ($year && $year <= format_date(time(), 'custom', 'Y')) {
    if (!is_null($month)) {
      if (!is_null($day)) {
        if ($last = gmdate('t', gmmktime(0, 0, 0, $month, 1, $year))) {
          $valid = (0 < $day && $day <= $last);
        }
      }
      else if (0 < $month && $month <= 12) { 
        $valid = TRUE;
      }
    }
    else {
      $valid = TRUE;
    }
  }
  return $valid;
}

/**
 * Check if given node type is valid for a Drupal archive
 *
 * @param $type
 *    The type to check
 * @return
 *    TRUE or FALSE
 */
function _archive_validate_type($type) {
  $types = variable_get('archive_type_filters', array());
  return in_array($type, $types);
}

/**
 * Generate an archive URL based on the $y, $m and $d
 * provided
 *
 * @param $type
 *    The node type to use if valid
 * @param $y
 *    The year to use if valid
 * @param $m
 *    The month to use if valid
 * @param $d
 *    The day to use if valid
 * @return
 *    A string with the generated archive URL
 */
function _archive_url($type, $y = 0, $m = 0, $d = 0) {
  $url = 'archive';
  if (_archive_validate_type($type)) {
    $url .= '/'. $type;
  }
  else {
    $url .= '/all';
  }
  
  if (_archive_validate_date($y, $m, $d)) {
    $url .= '/'. $y .'/'. $m .'/'. $d;
  }
  else if (_archive_validate_date($y, $m)) {
    $url .= '/'. $y .'/'. $m;
  }
  else if (_archive_validate_date($y)) {
    $url .= '/'. $y;
  }
  return $url;
}

/**
 * Determine timezone to use for the dates (from format_date)
 *
 * @return
 *   Timezone offset to use in time operations
 */
function _archive_get_timezone() {
  global $user;
  if (variable_get('configurable_timezones', 1) && $user->uid && strlen($user->timezone)) {
    return $user->timezone;
  }
  else {
    return variable_get('date_default_timezone', 0);
  }
}

/**
 * Theme the archive navigation with years, months and dates by default.
 * @ingroup themeable
 */
function theme_archive_navigation($type, $date) {
  $output  = "<div id=\"archive-container\"><dl><dt>". t('Date') ."</dt><dd>\n";
  $output .= theme('archive_navigation_years', $type, $date);
  if (_archive_validate_date($date->year)) {
    $output .= theme('archive_navigation_months', $type, $date);
  }
  if (_archive_validate_date($date->year, $date->month)) {
    $output .= theme('archive_navigation_days', $type, $date);
  }
  $output .= "</dd>";
  
  // Only display node type filter if more than one node type represented
  if (sizeof(_archive_node_types($date)) > 2) {
    $output .= "<dt>". t('Type') ."</dt><dd>\n";
    $output .= theme('archive_navigation_node_types', $type, $date);
    $output .= "</dd>";
  }
  $output .= "</dl></div>\n";
  return $output;
}

/**
 * Theme the list of years for the archive navigation.
 * @ingroup themeable
 */
function theme_archive_navigation_years($type, $date) {  
  $all_years = array_keys($date->years);
  $output = "<ul id=\"archive-years\">\n";
  
  $all_count = 0;
  foreach ($date->years as $year) {
    $all_count += $year;
  }
  
  $output .= '<li'. ($date->year?'':' class="selected"') .'>'. l('All', 'archive', array('attributes' => array('title' => format_plural($all_count, '1 post', '@count posts')))) ."</li>\n";
  for ($year = min($all_years); $year <= max($all_years); $year++) {
    $class = '';
    if ($year == $date->year) {
      $class = ' class="selected"';
    }
    $output .= '<li'. $class .'>'. l($year, _archive_url($type, $year)) ."</li>\n";
  }
  $output .= "</ul>\n";

  return $output;
}

/**
 * Theme the list of months for the archive navigation.
 * @ingroup themeable
 */
function theme_archive_navigation_months($type, $date) {
  $output = "<ul id=\"archive-months\">\n";
  
  $all_count = 0;
  foreach ($date->months as $month) {
    $all_count += $month;
  }
  
  $output .= '<li'. ($date->month?'':' class="selected"') .'>'. l('All', _archive_url($type, $date->year), array('attributes' => array('title' => format_plural($all_count, '1 post', '@count posts')))) ."</li>\n";
  $curr_month = date('n', time());
  $curr_year = date('Y', time());
  foreach (range(1, 12) as $month) {
    $posts = !empty($date->months[$month]) ? $date->months[$month] : 0;
    $class = '';
    if ($month == $date->month) {
      $class = ' class="selected"';
    }
    else if ($curr_year == $date->year && $month > $curr_month) {
      $class = ' class="future"';
    }
    $month_names = array('', 'Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec');
    $output .= "<li$class>". ($posts > 0 ? l($month_names[$month], _archive_url($type, $date->year, $month), array('attributes' => array('title' => format_plural($posts, "1 post", "@count posts")))) : $month_names[$month]) ."</li>\n";
  }
  $output .= "</ul>\n";
  return $output;
}

/**
 * Theme the list of days for the archive navigation.
 * @ingroup themeable
 */
function theme_archive_navigation_days($type, $date) {
  $output = "";
  $day_stop = gmdate('t', gmmktime(0, 0, 0, $date->month, 1, $date->year));
  $output = "<ul id=\"archive-days\">\n";
  
  $all_count = 0;
  foreach ($date->days as $day) {
    $all_count += $day;
  }
  
  $output .= '<li'. ($date->day?'':' class="selected"') .'>'. l('All', _archive_url($type, $date->year, $date->month), array('attributes' => array('title' => format_plural($all_count, '1 post', '@count posts')))) ."</li>\n";
  $curr_month = date('n', time());
  $curr_year = date('Y', time());
  $curr_day = date('j', time());
  for ($day = 1; $day <= $day_stop; $day++) {
    $posts = array_key_exists($day, $date->days) ? $date->days[$day] : 0;
    $class = '';
    if ($day == $date->day) {
      $class = ' class="selected"';
    }
    else if ($curr_year == $date->year && $curr_month == $date->month && $day > $curr_day) {
      $class = ' class="future"';
    }
    $output .= "<li$class>". ($posts ? l($day, _archive_url($type, $date->year, $date->month, $day), array('attributes' => array("title" => format_plural($posts, "1 post", "@count posts")))) : $day) ."</li>\n";
  }
  $output .= "</ul>\n";
  return $output;
}

/**
 * Theme the list of node types for the archives
 * @ingroup themeable
 */
function theme_archive_navigation_node_types($type, $date) {
  
  $output = "<ul id=\"archive-node_types\">\n";
  $types_count = _archive_node_types($date);
  
  $all_count = 0;
  foreach ($types_count as $t) {
    $all_count += $t['count'];
  }
  
  $output .= '<li'. ($type && $type != 'all'?'':' class="selected"') .'>'. l('All', _archive_url('all', $date->year, $date->month, $date->day), array('attributes' => array('title' => format_plural($all_count, '1 post', '@count posts')))) ."</li>\n";
  foreach ($types_count as $ft_key => $ft_value) {
    if (!$ft_value['count']) {
      continue;
    }
    $class = ($ft_key == $type ? ' class="selected"' : '');
    $name = $ft_value['name'];
    if ($types_count[$ft_key]['count'] > 0) {
      $output .= "<li$class>". l($name, _archive_url($ft_key, $date->year, $date->month, $date->day), array('attributes' => array("title" => format_plural($types_count[$ft_key]['count'], "1 post", "@count posts")))) ."</li>\n";
    }
    else {
      $output .= "<li$class>$name</li>\n";
    }
  }
  $output .= "</ul>\n";
  return $output;
}

/**
 * Theme the date separators between nodes of different year/month/day
 * Takes an UNIX timestamp and a string-indexed array
 * @ingroup themeable
 */
function theme_archive_separator($date_created, $separators) {
  $rv = '<h3>';
  if ($separators['month']) {
    $rv .= date('F', $date_created) .' ';
  }
  if ($separators['day']) {
    $rv .= date('jS', $date_created);
  }
  if ($separators['year']) {
    $rv .= ', '. date('Y', $date_created);
  }
  
  return $rv .'</h3>';
}
